pub struct Tree { /* private fields */ }Expand description
A binary search tree which maps cardinal values to ordinal values
Implementations
sourceimpl Tree
impl Tree
sourcepub fn with_capacity(capacity: usize) -> Self
pub fn with_capacity(capacity: usize) -> Self
Construct a new Tree with the given capacity.
sourcepub fn insert(&mut self, ordinal: usize) -> bool
pub fn insert(&mut self, ordinal: usize) -> bool
Insert an ordinal into this Tree and return false if it was already present.
Panics:
if ordinal >= self.max_size()